/*
 *  @file   FrameQBufMgr.c
 *
 *  @brief      Implements FrameQBufMgr functions.
 *
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */



/* Standard headers */
#include <ti/syslink/Std.h>

/* Utilities & OSAL headers */
#include <ti/syslink/utils/Trace.h>
#include <ti/syslink/utils/Memory.h>
#include <ti/syslink/utils/Gate.h>
#include <ti/syslink/utils/GateMutex.h>
#include <ti/syslink/utils/Cache.h>
#include <ti/syslink/utils/String.h>
#include <ti/syslink/utils/List.h>
#include <ti/syslink/utils/IGateProvider.h>
#include <ti/syslink/utils/IHeap.h>
#include <Bitops.h>

/* Utilities & OSAL headers */
#include <_MultiProc.h>
#include <ti/ipc/MultiProc.h>
/* Module level headers */
#include <ti/ipc/SharedRegion.h>
#include <ti/ipc/NameServer.h>
#include <_ListMP.h>
#include <ti/ipc/ListMP.h>

#include <ti/syslink/inc/ClientNotifyMgr.h>
#include <ti/syslink/FrameQDefs.h>

#include <ti/syslink/FrameQBufMgr.h>
#include <ti/syslink/FrameQBufMgr_ShMem.h>
#include <_FrameQBufMgr_ShMem.h>
#include <_FrameQBufMgr.h>

#include <IFrameQBufMgr.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */

/** ============================================================================
 *  @const  IPC_BUFFER_ALIGN
 *
 *  @desc   Macro to align a number.
 *          x: The number to be aligned
 *          y: The value that the number should be aligned to.
 *  ============================================================================
 */
#define IPC_BUFFER_ALIGN(x, y) (UInt32)((UInt32)((x + y - 1) / y) * y)

/*=============================================================================
 * Macros
 *=============================================================================
 */

/*! @brief Macro to make a correct module magic number with refCount */
#define FRAMEQBUFMGR_MAKE_MAGICSTAMP(x) ((FrameQBufMgr_MODULEID << 12u) | (x))

/* =============================================================================
 * Structures & Enums
 * =============================================================================
 */
/*
 * @brief Structure for FrameQBufMgr module state
 */
typedef struct FrameQBufMgr_ModuleObject_Tag {
    Atomic                      refCount;
    /*!< Reference count */
    NameServer_Handle           nameServer;
    /*!< Handle to the local NameServer used for stoting FrameQBufMgrBufMgr instance
     *   information.
     */
    UInt32                      nameServerType;
    /*!< Name server create type. Dynamic or static*/
    IGateProvider_Handle        gate;
    /*!< Handle for module wide protection*/
    Bool                        gateType;
    /*!< Flag denotes if gate is dynamically created/opened one */
    List_Object                 objList;
    /* List holding created objects */
    IGateProvider_Handle        listLock;
    FrameQBufMgr_Config         cfg;
    /*!<  Current module wide config values*/
    FrameQBufMgr_Config         defaultCfg;
    /*!< Default config values*/
    FrameQBufMgr_Params     defaultInstParams;
    /*!< Default instance creation parameters.
     *   Same for all the implementations.
     */
     Bool isInstNoValid[ClientNotifyMgr_maxInstances];
    /*  Create call will check this flags and sets first invalid index to valid
     *  and  assigns this index to the first byte of obj->Id
     */
} FrameQBufMgr_ModuleObject;

/*
 * @brief Structure defining internal object for the FrameQBufMgrBufMgr
 */
typedef struct FrameQBufMgr_Obj_Tag {
    List_Elem               listElem;
    /*!< Used for creating a linked list */
    Ptr                     handle;
    /* Handle to implementation specific instance */
    UInt32                  instId;
    /*!< Id of the instance*/
    UInt32                  notifyId;
    /*!<  Location to  store the key returned by the
     *    ClientNotiftmgr_registerClient call
     */
    UInt32                  objType;
    /*!<  Type of the object. Dynamic create, Dynamic open,
     *    static create or static open
     */
    UInt32                  ctrlInterfaceType;
    /*!<  Type of the interface */
    UInt32                  refCount;

    Char                    name[32];
    /*!< Name of the instance */
    Bool                    ctrlStructCacheFlag ;
    /*!<  Flag indicating whether Cache coherence API needs to be called for
     *    instance control structures
     */
    Bool                    frmHdrBufCacheFlag ;
    /*!<  Flag indicating whether Cache coherence API needs to be called for
     *    frame header buffers.
     */
    Bool                    frmBufCacheFlag[8] ;
    /*!<  Flag indicating whether Cache coherence API needs to be called for
     *    frame buffers
     */
    Bool                    bufCpuAccessFlag[8];
    /*!<  Flag indicating whether APP uses CPU to access these frame buffers.
     */
    Bool                    isRegistered ;
    /*!< Indicates if client has registered a call back function */
    Ptr                         top;
    /* Pointer to the top Object */
    FrameQBufMgr_Params          params;
    /*!< Instance  config params */
} FrameQBufMgr_Obj;

/*
 * @brief Structure defining object for the FrameQBufMgr.
 */
struct FrameQBufMgr_Object{
    Ptr obj;
    /*!< Pointer to the FrameQBufMgr internal object*/
   /*TODO: Add function table interface */
};

typedef struct FrameQBufMgr_Object FrameQBufMgr_Object;
/*=============================================================================
 * Globals
 *=============================================================================
 */
/*!
 *  @var    FrameQBufMgr_module_obj
 *
 *  @brief  FrameQBufMgr_module state variable.
 */
#if !defined(SYSLINK_BUILD_DEBUG)
static
#endif /* if !defined(SYSLINK_BUILD_DEBUG) */
FrameQBufMgr_ModuleObject FrameQBufMgr_module_obj = {
    .nameServer                   = NULL,
    .defaultCfg.eventNo           = FrameQBufMgr_NOTIFY_RESERVED_EVENTNO
};


/*!
 *  @var    FrameQBufMgr_module
 *
 *  @brief  Pointer to FrameQBufMgr_module_obj .
 */
#if !defined(SYSLINK_BUILD_DEBUG)
static
#endif /* if !defined(SYSLINK_BUILD_DEBUG) */
FrameQBufMgr_ModuleObject* FrameQBufMgr_module = &FrameQBufMgr_module_obj;

/*  NameServer handle for the FrameQBufMgr module */
Ptr gFrameQBufMgr_nsHandle;
/*=============================================================================
 * Forward declarations of internal Functions
 *=============================================================================
 */
inline Int32 FrameQBufMgr_isSupportedInterface(UInt32 type);

Void FrameQBufMgr_Params_init (FrameQBufMgr_Handle  handle,
                               FrameQBufMgr_Params * params);
static inline
Int32
_FrameQBufMgr_create (FrameQBufMgr_Object* obj,
                      FrameQBufMgr_Params* params);

static inline
Int32
 _FrameQBufMgr_open (FrameQBufMgr_Object* obj,
                     FrameQBufMgr_Params* params);

/*=============================================================================
 * APIs
 *=============================================================================
 */
/*!
 *  @brief  Function to initialize the config parameter structure  with default
 *          values.
 *
 *  @param  cfg
 *          Parameter structure to return the config parameters.
 *
 *  @sa    None.
 */
Int32
FrameQBufMgr_getConfig (FrameQBufMgr_Config * cfg)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    GT_0trace (curTrace, GT_ENTER, "FrameQBufMgr_getConfig");

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (cfg == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG  Argument of type
         *  (FrameQBufMgr_Config *) passed is null.
         */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_getConfig",
                             status,
                             "cfg argument is NULL");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                      FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                      FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
            == TRUE) {
            /* Return the default static config values */
            Memory_copy ((Ptr) cfg,
                          (Ptr) &(FrameQBufMgr_module->defaultCfg),
                          sizeof (FrameQBufMgr_Config));
        }
        else {
            Memory_copy ((Ptr) cfg,
                         (Ptr) &(FrameQBufMgr_module->cfg),
                         sizeof (FrameQBufMgr_Config));
        }
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_getConfig", status);

    return (status);
}

/*!
 *  @brief  Function to setup the FrameQBufMgr module.
 *
 *          This function sets up the FrameQBufMgr module. This function must
 *          be called before any other instance-level APIs can be invoked.
 *          Module-level configuration needs to be provided to this
 *          function. If the user wishes to change some specific config
 *          parameters, then FrameQBufMgr_getConfig can be called to get the
 *          configuration filled with the default values. After this, only
 *          the required configuration values can be changed. If the user
 *          does not wish to make any change in the default parameters, the
 *          application can simply call setup with NULL parameters.
 *          The default parameters would get automatically used.
 *  @param  cfg
 *          Optional FrameQBufMgr module configuration. If provided as
 *          NULL, default configuration is used.
 *
 *  @sa    None.
 */
Int32
FrameQBufMgr_setup (FrameQBufMgr_Config * cfg)
{
    Int32                  status = FrameQBufMgr_S_SUCCESS;
    Error_Block            eb;
    NameServer_Params      nameServerParams;
    Int32                  tmpStatus;
    FrameQBufMgr_Config    tmpCfg;

    GT_1trace (curTrace, GT_ENTER, "FrameQBufMgr_setup", cfg);
    Error_init (&eb);

    if (cfg == NULL) {
        FrameQBufMgr_getConfig (&tmpCfg);
        cfg = &tmpCfg;
    }

    if (cfg == NULL) {
        FrameQBufMgr_getConfig (&tmpCfg);
        cfg = &tmpCfg;
    }

    /* This sets the refCount variable is not initialized, upper 16 bits is
     * written with module Id to ensure correctness of refCount variable.
     */
    Atomic_cmpmask_and_set (&FrameQBufMgr_module->refCount,
                            FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                            FRAMEQBUFMGR_MAKE_MAGICSTAMP(0));

    if (   Atomic_inc_return (&FrameQBufMgr_module->refCount)
        != FRAMEQBUFMGR_MAKE_MAGICSTAMP(1u)) {
        status = FrameQBufMgr_S_ALREADYSETUP;
        GT_0trace (curTrace,
                   GT_2CLASS,
                   "FrameQBufMgr Module already initialized!");
    }
    else {
        /* Create a locate gate */
        FrameQBufMgr_module->gate = (IGateProvider_Handle)
                               GateMutex_create ((GateMutex_Params*)NULL, &eb);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
        if (FrameQBufMgr_module->gate == NULL) {
             GT_assert(curTrace, (FrameQBufMgr_module->gate != NULL));
             status = FrameQBufMgr_E_CREATE_GATEMP ;
             GT_setFailureReason (curTrace,
                                 GT_4CLASS,
                                 "FrameQBufMgr_setup",
                                 status,
                                 "GateMutex_create()Failed to create module"
                                 "gate!");
        }
        else  {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
           /* Check the params required to create the name server instance */
            if (FrameQBufMgr_module->nameServer == NULL) {
                /* Create NameServer.
                 * Let NameServer create a gate for it
                 */
                /* Get the default params for  the Name server */
                NameServer_Params_init(&nameServerParams);
                /* Update the nameServerParams  as per FrameQ requirements */
                nameServerParams.maxRuntimeEntries = cfg->maxInstances; /* max instances on this processor */
                nameServerParams.checkExisting     = TRUE; /* Check if exists */
                /* Length of  the FrameQ instance */
                nameServerParams.maxNameLen        = FrameQBufMgr_MAXNAMELEN;
                nameServerParams.maxValueLen       =
                                         sizeof (FrameQBufMgr_NameServerEntry);

                /* Create the Name Server instance */
                FrameQBufMgr_module->nameServer = NameServer_create(
                                                    FRAMEQBUFMGR_NAMESERVERNAME,
                                                    &nameServerParams);
#if !defined (SYSLINK_BUILD_OPTIMIZE)
                if(NULL == FrameQBufMgr_module->nameServer) {
                    status = FrameQBufMgr_E_CREATE_NAMESERVER;
                    /*! @retval FrameQBufMgr_E_CREATE_NAMESERVER
                     * Failed to create NameServer */
                    GT_setFailureReason (curTrace,
                                         GT_4CLASS,
                                         "FrameQBufMgr_setup",
                                         status,
                                         "Failed to create NameServer!");
                    /* Decrement the reference count */
                    Atomic_dec_return (&FrameQBufMgr_module->refCount);
                }
                else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
                    gFrameQBufMgr_nsHandle = FrameQBufMgr_module->nameServer;
                    FrameQBufMgr_module->nameServerType =
                                                FrameQBufMgr_DYNAMIC_CREATE;
#if !defined (SYSLINK_BUILD_OPTIMIZE)
                }
#endif
            }
#if !defined(SYSLINK_BUILD_OPTIMIZE)
        }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

        if (status >= 0) {
            /* Construct the list object */
            List_construct (&FrameQBufMgr_module->objList, NULL);
            /* Copy the cfg */
            Memory_copy ((Ptr) &FrameQBufMgr_module->cfg,
                         (Ptr) cfg,
                         sizeof (FrameQBufMgr_Config));
            /* Call all the implementataion specific setups */
            tmpStatus = FrameQBufMgr_ShMem_setup(cfg);
            if (tmpStatus < 0) {
                status = tmpStatus;
            }
        }
    }

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_setup", status);

    /*!< FrameQBufMgr_S_SUCCESS if module setup is done successfully */
    return (status);
}


/*!
 *  @brief  Function to destroy(finalize) the FrameQBufMgr module.
 *
 */
Int32
FrameQBufMgr_destroy (Void)
{
    Int32 status = FrameQBufMgr_S_SUCCESS;
    Int32 tmpStatus;

    GT_0trace (curTrace, GT_ENTER, "FrameQBufMgr_destroy");

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval FrameQBufMgr_E_INVALIDSTATE Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_destroy",
                             status,
                             "Module was not initialized!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        if (   Atomic_dec_return (&FrameQBufMgr_module->refCount)
            == FRAMEQBUFMGR_MAKE_MAGICSTAMP(0)) {

            /* TODO: Delete instances which are not finalized */


            /* Call all the implementation specific destroys */
            tmpStatus = FrameQBufMgr_ShMem_destroy();
            if (tmpStatus < 0) {
                status = tmpStatus;
            }
            /* Delete  name server instance created  for this module,
             * if it is dynamically created.
             */
            if (   (FrameQBufMgr_module->nameServer != NULL)
                && (    FrameQBufMgr_module->nameServerType
                    ==  FrameQBufMgr_DYNAMIC_CREATE)) {
                NameServer_delete (&FrameQBufMgr_module->nameServer);
                FrameQBufMgr_module->nameServer = NULL;
            }

            /* Destruct the list object */
            List_destruct (&FrameQBufMgr_module->objList);

            if (FrameQBufMgr_module->gate != NULL) {
                /* Delete the module gate. */
                status = GateMutex_delete ((GateMutex_Handle*)&FrameQBufMgr_module->gate);
                GT_assert (curTrace, (status >= 0));
            }
            /* Clear cfg area */
            Memory_set ((Ptr) &FrameQBufMgr_module->cfg,
                        0,
                        sizeof (FrameQBufMgr_Config));
        }
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_destroy", status);

    /*! @retval FrameQBufMgr_S_SUCCESS Operation successful */
    return (status);
}

/*!
 * @brief Function to create a FrameQBufMgr instance
 *
 * @param  params Instance param structure.
 */
FrameQBufMgr_Handle
FrameQBufMgr_create (Ptr vparams)
{
    Int32                         status         = FrameQBufMgr_S_SUCCESS;
    Int32                         tmpStatus;
    IArg                          key0;
    FrameQBufMgr_Handle           handle         = NULL;
    FrameQBufMgr_Obj             *obj            = NULL;
    FrameQBufMgr_Params          *params = (FrameQBufMgr_Params*)vparams;

    GT_1trace (curTrace, GT_ENTER, "FrameQBufMgr_create", params);

    GT_assert (curTrace, (params != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
         /*! @retval NULL  Module was not initialized!
          */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_create",
                             status,
                             "Module was not initialized!");
    }
    else if ( NULL == params) {
         /*! @retval NULL  Params passed is NULL
          */
         status = FrameQBufMgr_E_INVALIDARG;
         GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_create",
                             status,
                             "Params pointer is NULL!");

    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        /* Create the handle */
        /* Allocate memory for handle */
        handle = (FrameQBufMgr_Handle) Memory_calloc (NULL,
                                                   sizeof (FrameQBufMgr_Object),
                                                   0u,
                                                   NULL);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
        if (handle == NULL) {
            /*! @retval NULL  Failed to allocate
             * memory for the instance handle.
             */
            status = FrameQBufMgr_E_ALLOC_MEMORY;
            GT_setFailureReason (curTrace,
                                 GT_4CLASS,
                                 "FrameQBufMgr_create",
                                 FrameQBufMgr_E_ALLOC_MEMORY,
                                 "Memory allocation failed for handle!");
        }
        else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
            obj = (FrameQBufMgr_Obj *) Memory_calloc (NULL,
                                                      sizeof (FrameQBufMgr_Obj),
                                                      0u,
                                                      NULL);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
            if (obj == NULL) {
                /*! @retval NULL  Failed to allocate
                 * memory for  instance internal object.
                 */
                status = FrameQBufMgr_E_ALLOC_MEMORY;
                Memory_free (NULL, handle, sizeof (FrameQBufMgr_Object));
                handle = NULL;
                GT_setFailureReason (curTrace,
                                     GT_4CLASS,
                                     "FrameQBufMgr_create",
                                     FrameQBufMgr_E_ALLOC_MEMORY,
                                     "Memory allocation failed for internal "
                                     "object!");
            }
            else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
                handle->obj = obj;
                obj->top    = handle;
                if (params->commonCreateParams.openFlag == TRUE) {
                   tmpStatus = _FrameQBufMgr_open (handle, params);
                   if (tmpStatus < 0) {
                        /*! @retval NULL  Failed to open the instance.
                         */
                        status = tmpStatus;
                        GT_setFailureReason (curTrace,
                                             GT_4CLASS,
                                             "FrameQBufMgr_create",
                                             status,
                                             "Failed to open the instance !");
                   }
                }
                else if (params->commonCreateParams.openFlag == FALSE) {
                    tmpStatus = _FrameQBufMgr_create (handle, params);
                    if (tmpStatus < 0) {
                        /*! @retval NULL  Failed to create the instance.
                         */
                        status = tmpStatus;
                        GT_setFailureReason (curTrace,
                                             GT_4CLASS,
                                             "FrameQBufMgr_create",
                                             status,
                                             "Failed to create instance !");
                   }
                }
#if !defined(SYSLINK_BUILD_OPTIMIZE)
            }
        }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

        if (status >= 0) {
            /* Populate the params member */
            Memory_copy ((Ptr) &obj->params,
                         (Ptr) params,
                         sizeof (FrameQBufMgr_Params));
             /* Put in the local list */
             key0 = IGateProvider_enter (FrameQBufMgr_module->gate);
             List_elemClear (&obj->listElem);
             List_put ((List_Handle) &FrameQBufMgr_module->objList,
                       &(obj->listElem));
             IGateProvider_leave (FrameQBufMgr_module->gate, key0);
        }
#if !defined(SYSLINK_BUILD_OPTIMIZE)
        /* Failed to crate /open the instance. Do clean up*/
        else {
            if (obj != NULL ) {
                Memory_free (NULL, obj, sizeof (FrameQBufMgr_Obj));
            }
            if (handle != NULL) {
                Memory_free (NULL, handle, sizeof (FrameQBufMgr_Object));
                handle = NULL;
            }
        }
    }
#endif

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_create", handle);

    /*! @retval Valid-Handle Operation successful*/
    return (handle);
}

/*!
 * @brief  Function to delete the created FrameQBufMgr instance
 *
 * @param  pHandle Pointer to the handle
 */
Int32
FrameQBufMgr_delete (FrameQBufMgr_Handle * pHandle)
{
    Int32                         status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Handle           handle;
    IArg                          key0;
    FrameQBufMgr_Obj             *obj;

    GT_1trace (curTrace, GT_ENTER, "FrameQBufMgr_delete", pHandle);

    GT_assert (curTrace, (pHandle != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval FrameQBufMgr_E_INVALIDSTATE Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_delete",
                             status,
                             "Module was not initialized!");
    }
    else if (pHandle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL*/
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_delete",
                             status,
                             "pHandle passed is NULL!");
    }
    else if (*pHandle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL*/
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_delete",
                             status,
                             "*pHandle passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        handle = *pHandle;
        obj = (FrameQBufMgr_Obj *) handle->obj;
        GT_assert (curTrace, (NULL != obj));

        switch (obj->objType) {

        case FrameQBufMgr_DYNAMIC_OPEN:
        {
            key0 = IGateProvider_enter (FrameQBufMgr_module->gate);
            /* Remove it from the local list */
            List_remove ((List_Handle) &FrameQBufMgr_module->objList,
                         &obj->listElem);
            IGateProvider_leave (FrameQBufMgr_module->gate, key0);

            /* Close the implementation specific handle and delete the top level
             * Object.
             */
            if (obj->handle != NULL) {
                switch (obj->ctrlInterfaceType) {
                    case FrameQBufMgr_INTERFACE_SHAREDMEM:
                    {
                        FrameQBufMgr_ShMem_close(
                                      (FrameQBufMgr_ShMem_Handle*)&obj->handle);
                    }
                    break;

                    default:
                    break;
                }
            }
            /* Now free the handle */
            Memory_free (NULL, obj, sizeof (FrameQBufMgr_Obj));
            obj = NULL;
            Memory_free (NULL, handle, sizeof (FrameQBufMgr_Object));
            handle = NULL;
            break;
        }

        case FrameQBufMgr_DYNAMIC_CREATE:
        {
            key0 = IGateProvider_enter (FrameQBufMgr_module->gate);
            /* Remove it from the local list */
            List_remove ((List_Handle) &FrameQBufMgr_module->objList,
                         &obj->listElem);
            IGateProvider_leave (FrameQBufMgr_module->gate, key0);
            /* Delete the implementation specific handle and delete the top
             * level Object.
             */
            if (obj->handle != NULL) {
                IFrameQBufMgr_delete((IFrameQBufMgr_Handle*)&obj->handle);
            }

            /* Now free the handle */
            Memory_free (NULL, obj, sizeof (FrameQBufMgr_Obj));
            obj = NULL;
            Memory_free (NULL, handle, sizeof (FrameQBufMgr_Object));
            handle = NULL;


            break;
        }

       default:
            status = FrameQBufMgr_E_ACCESSDENIED;
            /*! @retval FrameQBufMgr_E_ACCESSDENIED  Instance is not dynamically
             * created or opened .
             */
            GT_setFailureReason (curTrace,
                                 GT_4CLASS,
                                 "FrameQBufMgr_delete",
                                 status,
                                 "Invalid Handle!");
            break;
        }
#if !defined (SYSLINK_BUILD_OPTIMIZE)
    }
#endif
    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_delete", status);

    /*! @retval FrameQBufMgr_S_SUCCESS Operation successful */
    return (status);
}


/*!
 *  @brief  Function to open the created FrameQBufMgr instance.The instance must be
 *          created before openinig it.
 *
 *  @param  handlePtr
 *          Location to receive the created FrameQBufMgr instance.
 *  @param  intOpenParams
 *          open parameters specific to implementation. FrameQ_ShMem_OpenParams
 *          in case the instance is of type ShMem.
 *
 *
 */
Int32
FrameQBufMgr_open(FrameQBufMgr_Handle     *handlePtr,
                  Ptr                      instOpenParams)
{

    Int32                        status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Handle          frmQBufMgrHandle = NULL;
    Ptr                          impHandle = NULL;
    UInt32                       instId    = (UInt32)-1;
    FrameQBufMgr_Obj           * obj       = NULL;
    FrameQBufMgr_Params          params;
    FrameQBufMgr_NameServerEntry entry;
    FrameQBufMgr_NameServerEntry *pentry;
    UInt32                       ctrlInterfaceType;
    UInt32                       bufInterfaceType;
    UInt8                        nameLen;
    IArg                         key;
    UInt32                       len;
    UInt32                       index;
    FrameQBufMgr_OpenParams      *openParams = (FrameQBufMgr_OpenParams*)
                                                                 instOpenParams;

    GT_2trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_open",
               handlePtr,
               instOpenParams);

    GT_assert (curTrace, (handlePtr != NULL));
    GT_assert (curTrace, (instOpenParams != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                 FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                 FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval FrameQBufMgr_E_INVALIDSTATE Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_open",
                             status,
                             "Module was not initialized!");
    }
    else if (handlePtr == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG
         * Argument of type (FrameQBufMgr_Handle *) is NULL
         */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_open",
                             FrameQBufMgr_E_INVALIDARG,
                             "Argument of type (FrameQBufMgr_Handle *) is "
                             "NULL!");
    }
    else if (instOpenParams == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG
         * instOpenParams passed is null.
         */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_open",
                             status,
                             "instOpenParams passed is null.");
    }
    else if (   (String_len(openParams->commonOpenParams.name) == 0)
              &&(openParams->commonOpenParams.sharedAddr == NULL)) {
        /*! @retval FrameQBufMgr_E_INVALIDARG
         * Either name or sharedaddr must be non null.
         */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_open",
                             status,
                             "Either name or sharedaddr must be non null..");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        nameLen = strlen((String)openParams->commonOpenParams.name);

        if (   ( (nameLen > 0) && ( nameLen <= (FrameQBufMgr_MAXNAMELEN-1))
            && (NULL != FrameQBufMgr_module->nameServer))) {
            len = sizeof (FrameQBufMgr_NameServerEntry);
            status = NameServer_get (FrameQBufMgr_module->nameServer,
                                     (String)openParams->commonOpenParams.name,
                                     &entry,
                                     &len,
                                     NULL);
            if (status < 0) {
                status = FrameQBufMgr_E_NOTFOUND;
            }
            else {
                ctrlInterfaceType = entry.ctrlInterfaceType;
                bufInterfaceType  = entry.bufInterfaceType;
            }
        }
        else if (openParams->commonOpenParams.sharedAddr != NULL) {
            pentry = (FrameQBufMgr_NameServerEntry*)
                                       openParams->commonOpenParams.sharedAddr;
            /* Assert that sharedAddr is cache aligned */
            index = SharedRegion_getId(pentry);

            /* Assert that the buffer is in a valid shared region */
            GT_assert(curTrace, index != SharedRegion_INVALIDREGIONID);

            GT_assert(curTrace, ((UInt32)pentry %
                          SharedRegion_getCacheLineSize(index)) == 0);

            if (SharedRegion_isCacheEnabled(index))  {
                /* Invalidating the entry located in shared memory. The memory
                 * provided by this entry should be in cache aligend size
                 */
                key = Gate_enterSystem();
                Cache_inv (pentry,
                           sizeof (FrameQBufMgr_NameServerEntry),
                           Cache_Type_ALL,
                           TRUE);
                Gate_leaveSystem(key);
            }
            ctrlInterfaceType = pentry->ctrlInterfaceType;
            bufInterfaceType  = pentry->bufInterfaceType;
        }
        else {
            status = FrameQBufMgr_E_INVALIDARG;
            /*! @retval FrameQBufMgr_E_INVALIDARG
             * Either name or sharedaddr must be non null.
             */
            status = FrameQBufMgr_E_INVALIDARG;
            GT_setFailureReason (curTrace,
                                 GT_4CLASS,
                                 "FrameQBufMgr_open",
                                 status,
                                 "Either name or sharedaddr must be non null.");
        }

        if (status >= 0) {
             /* Check the interface type and if it is not supported
              * return failure.
              */
            if( !FrameQBufMgr_isSupportedInterface(ctrlInterfaceType)) {
                /*! @retval FrameQBufMgr_E_INVALID_INTERFACE
                 * Unsupported interface type.
                 */
                status = FrameQBufMgr_E_INVALID_INTERFACE;
                GT_setFailureReason (curTrace,
                                     GT_4CLASS,
                                     "FrameQBufMgr_open",
                                     status,
                                     "Unsupported interface type.");
            }
            else if(!FrameQBufMgr_isSupportedBufInterface(ctrlInterfaceType,
                                                          bufInterfaceType)) {
                /*! @retval FrameQBufMgr_E_INVALID_INTERFACE
                 * Unsupported interface type.
                 */
                status = FrameQBufMgr_E_INVALID_BUFINTERFACETYPE;
                GT_setFailureReason (curTrace,
                                     GT_4CLASS,
                                     "FrameQBufMgr_open",
                                     status,
                                     "Unsupported buf interface type.");
            }
            else {
                /* Get the default params */
                FrameQBufMgr_Params_init (NULL,&params);
                /* Name is string pointer in CreateParams*/
                params.commonCreateParams.name = (String)
                                             openParams->commonOpenParams.name;
                /* Opening the instance  */
                params.commonCreateParams.openFlag = TRUE;
                /* Call create in open mode to simply get the top level frame
                 * QBufMgr handle.
                 */
                frmQBufMgrHandle = FrameQBufMgr_create (&params);
                if (frmQBufMgrHandle == NULL) {
                    status = FrameQBufMgr_E_FAIL;
                }
                else {
                    /* Open handle to the implementation specific handle
                     * and plug in to the top level FrameQ object.
                     */
                    switch (ctrlInterfaceType) {
                        case FrameQBufMgr_INTERFACE_SHAREDMEM:
                            {
                                status = FrameQBufMgr_ShMem_open(
                                        (FrameQBufMgr_ShMem_Handle *)&impHandle,
                                         (FrameQBufMgr_ShMem_OpenParams*)openParams);
                                if ( (status < 0) || (impHandle == NULL)) {
                                    FrameQBufMgr_close(&frmQBufMgrHandle);
                                }
                                else {
                                    /* Get the instance id */
                                    instId = FrameQBufMgr_ShMem_getId(impHandle);
                                }
                            }
                        break;

                        default:
                            /* NameServer entry does not contain the
                             * implementation type.
                             */
                            status = FrameQBufMgr_E_INVALID_INTERFACE;
                   }
                }

            }
        }

        if (status >= 0) {
            /* obj is inside FrameQBufMgr handle */
            obj                = frmQBufMgrHandle->obj;
            obj->handle        = impHandle;
            obj->ctrlInterfaceType = ctrlInterfaceType;
            obj->objType       = FrameQBufMgr_DYNAMIC_OPEN;
            obj->instId        = instId;
            *handlePtr         = frmQBufMgrHandle;
        }
        else {
           *handlePtr         = NULL;
        }

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    /*! @retval FrameQBufMgr_S_SUCCESS Instance open successful*/
    return (status);
}


/*!
 * @brief Function to close the dynamically opened instance
 *
 * @param Instance handle.
 */
Int32
FrameQBufMgr_close(FrameQBufMgr_Handle * pHandle)
{
    Int32 status = FrameQBufMgr_S_SUCCESS;

    GT_1trace (curTrace, GT_ENTER, "FrameQBufMgr_close", pHandle);

    GT_assert (curTrace, (pHandle != NULL));
    GT_assert (curTrace, ((pHandle != NULL) && (*pHandle != NULL)));

    status = FrameQBufMgr_delete (pHandle);

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_close", status);

    return (status);
}

/*!
 *  @brief  Function to allocate frame from free frame queue zero.
 *
 *  @param  handle  FrameQBugMgr instance handle.
 *  @param  framePtr Location to receive the pointer to the allocated frame.
 */
Int32
FrameQBufMgr_alloc (FrameQBufMgr_Handle handle,
                    FrameQBufMgr_Frame  *framePtr)
{
    Int32                   status           = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj        *obj ;

    GT_2trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_alloc",
               handle,
               framePtr);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (framePtr != NULL));


#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval FrameQBufMgr_E_INVALIDSTATE if Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_alloc",
                             status,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_alloc",
                             status,
                             "handle passed is NULL!");
    }
    else if (framePtr == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG buf passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_alloc",
                             status,
                             "framePtr passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, (obj != NULL));

        /* Call implementation specific alloc */
        status = IFrameQBufMgr_alloc (obj->handle, framePtr);


#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_alloc", status);

    /*! @retval FrameQBufMgr_S_SUCCESS if buffer is allocated successfully */
    return (status);
}


/*!
 *  @brief  Function to allocate multiple frames from the FrameQbufMgr instance.
 *
 *  @param  handle   Instance handle.
 *  @param  framePtr Array to receive pointers to allocated frames.
 *  @param  freeQId  Array of free frame pool nos from which API needs to
                     allocate frames actual size of that is allocated.
 *  @param  numFrames Number of frames that needs to be allocated. When API
 *                    returns it holds the actual number of frames allocated.
 *
 */
Int32
FrameQBufMgr_allocv (FrameQBufMgr_Handle  handle ,
                     FrameQBufMgr_Frame   framePtr[],
                     UInt32               freeQId[],
                     UInt8                     *numFrames)
{
    Int32                              status           = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj                  *obj ;

    GT_4trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_allocv",
               handle,
               framePtr,
               freeQId,
               numFrames);

    GT_assert (curTrace,(handle != NULL));
    GT_assert (curTrace,(framePtr != NULL));
    GT_assert (curTrace,(freeQId != NULL));
    GT_assert (curTrace,(numFrames != NULL));
    GT_assert (curTrace,(FrameQBufMgr_MAX_POOLS > *numFrames));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval FrameQBufMgr_E_INVALIDSTATE if Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_allocv",
                             status,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_allocv",
                             status,
                             "handle passed is NULL!");
    }
    else if (framePtr == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG framePtr passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_allocv",
                             status,
                             "framePtr passed is NULL!");
    }
    else if (freeQId == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG freeQId pointer passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_allocv",
                             status,
                             "freeQId passed is NULL!");
    }
    else if (numFrames == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG freeQId pointer passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_allocv",
                             status,
                             "numFrames passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, (obj != NULL));

        status = IFrameQBufMgr_allocv (obj->handle,
                                       framePtr,
                                       freeQId,
                                       numFrames);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_allocv", status);

   /*! @retval FrameQBufMgr_S_SUCCESS if number of buffers specified are
    * allocated
    */
    return (status);

}

/*!
 *  @brief  Function to free frame.
 *
 *          This API picks up the freeFrmQueueNo from the frame header and
 *          consider  that frame pool to free the given frame.
 *
 *  @param  handle  FrameQBugMgr instance handle.
 *  @param  frame   Frame to be freed.
 *
 */
Int32
FrameQBufMgr_free (FrameQBufMgr_Handle handle,
                   FrameQBufMgr_Frame   frame)
{
    Int32                         status        = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj             *obj;

    GT_2trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_free",
               handle,
               frame);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (frame != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval FrameQBufMgr_E_INVALIDSTATE if Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_free",
                             status,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_free",
                             status,
                             "handle passed is NULL!");
    }
    else if (frame == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG frame passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_free",
                             status,
                             "frame passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, (obj != NULL));

        status = IFrameQBufMgr_free (obj->handle, frame);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

     GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_free", status);

     /*! @retval FrameQBufMgr_S_SUCCESS if buffer is freed successfully.
      */
     return (status);
}

/*!
 *  @brief  Function to free multiple frames
 *
 *          Function to free multiple frames  to the  FrameQBufMgr.
 *          Notification to other clients will be generated once all the frames
 *          are freed.
 *
 *  @param  handle      FrameQBugMgr instance handle.
 *  @param  framePtr    Array of frames to be freed.
 *  @param  numFrames   Number of frames to be freed.
 */
Int32
FrameQBufMgr_freev (FrameQBufMgr_Handle  handle,
                    FrameQBufMgr_Frame   framePtr[],
                    UInt32               numFrames)
{
    Int32                         status        = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj             *obj;

    GT_3trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_freev",
               handle,
               framePtr,
               numFrames);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (framePtr != NULL));
    GT_assert (curTrace, (numFrames != 0));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval FrameQBufMgr_E_INVALIDSTATE if Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_freev",
                             status,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_freev",
                             status,
                             "handle passed is NULL!");
    }
    else if (framePtr == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG frame passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_freev",
                             status,
                             "framePtr passed is NULL!");
    }
    else if (numFrames == 0) {
        /*! @retval FrameQBufMgr_E_INVALIDARG numFrames passed is zero */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_freev",
                             status,
                             "numFrames passed is zero!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, (obj != NULL));

        status = IFrameQBufMgr_freev (obj->handle, framePtr, numFrames);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_freev", status);

    /*! @retval FrameQBufMgr_S_SUCCESS if buffer is freed successfully.
     */
    return (status);
}

/*!
 *  @brief  Function to add  free frames dynamically  to the FrameQBufMgr
 *          instance
 *
 *  @param  handle      FrameQBugMgr instance handle.
 *  @param  freeQId     Free  frame pool no.
 *  @param  framePtr    frame to be added.
 *
 */
Ptr
FrameQBufMgr_add (FrameQBufMgr_Handle  handle,
                  UInt8                freeQId)
{
    Ptr                 ptr = NULL;
    FrameQBufMgr_Obj   *obj;


    GT_2trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_add",
               handle, freeQId);

    GT_assert (curTrace, (NULL != handle));
    GT_assert (curTrace, (freeQId < FrameQBufMgr_MAX_POOLS));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_add",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (freeQId >= FrameQBufMgr_MAX_POOLS) {
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_add",
                             FrameQBufMgr_E_INVALIDARG,
                             "Invalid freeQId provided for buf!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;

        ptr = IFrameQBufMgr_add (obj->handle, freeQId);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_add", ptr);

    /*! @retval Frame if frame is added successfully */
    return (ptr);
}

/*!
 *  @brief  Function to remove the added  free frames from the FrameQBufMgr
 *          instance
 *
 *  @param  handle      FrameQBugMgr instance handle.
 *  @param  freeQId     Free  frame pool no.
 *  @param  framePtr    frame to be removed.
 *
 */
Int32
FrameQBufMgr_remove (FrameQBufMgr_Handle  handle,
                     UInt8                freeQId,
                     FrameQBufMgr_Frame   framePtr)
{
    Int32               status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj   *obj;

    GT_3trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_remove",
               handle, freeQId, framePtr);

    GT_assert (curTrace, (NULL != handle));
    GT_assert (curTrace, (NULL != framePtr));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_remove",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (framePtr == NULL) {
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_remove",
                             FrameQBufMgr_E_INVALIDARG,
                             "Invalid value NULL provided for framePtr!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;

        status = IFrameQBufMgr_remove (obj->handle, freeQId, framePtr);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_remove", status);

    /*! @retval FrameQBufMgr_S_SUCCESS if buffer added successfully */
    return (status);
}

/*!
 *  @brief  Function to Register notification with the instance.
 *
 *          It internally registers notification with the individual free queues
 *          (free frame pools). If alloc on free pool 0 failed,Notification
 *          will be generated if free frames in that pool becomes more than
 *          watermark(all the free frames in that pool).
 *
 *  @param  handle          FrameQBugMgr instance handle.
 *  @param  notifyParams    notification registration parameters.
 */
Int32
FrameQBufMgr_registerNotifier (FrameQBufMgr_Handle        handle,
                               FrameQBufMgr_NotifyParams *notifyParams)
{
    Int32                                  status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj  *obj;

    GT_2trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_registerNotifier",
               handle,
               notifyParams);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (notifyParams != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval FrameQBufMgr_E_INVALIDSTATE if Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_registerNotifier",
                             status,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_registerNotifier",
                             status,
                             "handle passed is NULL!");
    }
    else if (notifyParams == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG notifyParams passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_registerNotifier",
                             status,
                             "notifyParams passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, (obj != NULL));
        status = IFrameQBufMgr_registerNotifier (obj->handle, notifyParams);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace,
               GT_LEAVE,
               "FrameQBufMgr_registerNotifier",
               status);

    /*! @retval FrameQBufMgr_S_SUCCESS if register notifier done successfully.
     */
    return (status);
}


/*!
 * @brief Function to unregister the notification call back function.
 *
 * @param  handle Handle to the FrameQBufMgr instance
 */
Int32
FrameQBufMgr_unregisterNotifier (FrameQBufMgr_Handle handle)
{
    Int32              status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj  *obj;

    GT_1trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_unregisterNotifier",
               handle);

    GT_assert (curTrace, (handle != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval FrameQBufMgr_E_INVALIDSTATE if Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_unregisterNotifier",
                             status,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_unregisterNotifier",
                             status,
                             "handle passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);

        status = IFrameQBufMgr_unregisterNotifier (obj->handle);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace,
               GT_LEAVE,
              "FrameQBufMgr_unregisterNotifier",
              status);

    /*! @retval FrameQBufMgr_S_SUCCESS if callback function unregistered
     *  successfully.
     */
    return (status);
}

/*!
 * @brief   API to duplicate the given frame i.e it increments the corresponding
 *          frames 's reference count.It internally allocates  headers for the
 *          duped frames.It fails if it is not able to dup all the frames.
 *  @param  frame           Frame to duplicated.
 *  @param  dupedFramePtr   Location to receive duplicated frames.
 *  @param  numDupedFrames  Number of duplicated frames required.
 */
Int32
FrameQBufMgr_dup (FrameQBufMgr_Handle  handle,
                  FrameQBufMgr_Frame   frame,
                  FrameQBufMgr_Frame   dupedFramePtr[],
                  UInt32               numDupedFrames)
{
    Int32   status       = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj    *obj;

    GT_4trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_dup",
               handle,
               frame,
               dupedFramePtr,
               numDupedFrames);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (frame != NULL));
    GT_assert (curTrace, (dupedFramePtr != NULL));
    GT_assert (curTrace, (numDupedFrames != 0));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval NULL if Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_dup",
                             status,
                             "Module was not initialized!");
    }
    else if (frame == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG frame to be duplicated is NULL */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_dup",
                             status,
                             "frame to be duplicated is NULL!");
    }
    else if (dupedFramePtr == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG Location to receive
         *  duplicated frames is NULL
         */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_dup",
                             status,
                             "Location to receive duplicated frames is NULL !");
    }
    else if (numDupedFrames == 0) {
        /*! @retval FrameQBufMgr_E_INVALIDARG numDupedFrames passed is zero */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_dup",
                             status,
                             "numDupedFrames passed is 0!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, (obj != NULL));
        status = IFrameQBufMgr_dup (obj->handle,
                                    frame,
                                    dupedFramePtr,
                                    numDupedFrames);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_dup", status);

    /*! @retval FrameQBufMgr_S_SUCCESS if buffer duplicated successfully.
     */
    return (status);
}

/*!
 * @brief   API to duplicate the given frames i.e it increments the
 *          corresponding frames 's reference count.It internally allocates
 *          headers for the duped frames.API fails if it is not able to dup all
 *          the frames.
 * @param  framePtr         Array of frames to duplicated.
 * @param  dupedFramePtr    Location to receive duplicated frames.
 * @param  numDupedFrames   Number of duplicated frames required for each  given
 *                          frame.
 * @param  numFrames        Number of original frames that needs to be
 *                          duplicated.
 */
Int32
FrameQBufMgr_dupv (FrameQBufMgr_Handle   handle,
                   FrameQBufMgr_Frame    framePtr[],
                   FrameQBufMgr_Frame  **dupedFramePtr,
                   UInt32                numDupedFrames,
                   UInt32                numFrames)
{
    Int32       status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj    *obj;

    GT_5trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_dupv",
               handle,
               framePtr,
               dupedFramePtr,
               numDupedFrames,
               numFrames);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (framePtr != NULL));
    GT_assert (curTrace, (dupedFramePtr != NULL));
    GT_assert (curTrace, (numDupedFrames != 0));
    GT_assert (curTrace, (numFrames != 0));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval NULL if Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_dupv",
                             status,
                             "Module was not initialized!");
    }
    else if (framePtr == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG Array of frames to be duplicated
         * is NULL
         */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_dupv",
                             status,
                             "Array of frames to be duplicated is NULL!");
    }
    else if (dupedFramePtr == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG Location to receive
         *  duplicated frames is NULL
         */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_dupv",
                             status,
                             "Location to receive duplicated frames is NULL !");
    }
    else if (numDupedFrames == 0) {
        /*! @retval FrameQBufMgr_E_INVALIDARG numDupedFrames passed is zero */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_dupv",
                             status,
                             "numDupedFrames passed is zero!");
    }
    else if (numFrames == 0) {
        /*! @retval FrameQBufMgr_E_INVALIDARG numFrames passed is zero */
        status = FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_dupv",
                             status,
                             "numFrames passed is zero!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, (obj != NULL));
        status = IFrameQBufMgr_dupv (obj->handle,
                                     framePtr,
                                     dupedFramePtr,
                                     numDupedFrames,
                                     numFrames);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_dupv", status);

    /*! @retval FrameQBufMgr_S_SUCCESS if buffer duplicated successfully.
     */
    return (status);
}

/*!
 *  @brief  Write back the contents  of a frame including frame buffers..
 *
 *          NOTE: frame address and frame buffer address in the frame should be in
 *          virtual format.
 *  @param  handle  Handle to the instance.
 *  @param  frame   Frame to be writeback.
 */
Int32
FrameQBufMgr_writeBack(FrameQBufMgr_Handle  handle,
                       FrameQBufMgr_Frame   frame)
{
    Int32   status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj  *obj;

    GT_2trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_writeBack",
               handle,
               frame);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (frame  != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval 0 Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_writeBack",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status =  FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_writeBack",
                             status,
                             "handle passed is NULL!");
    }
    else if (frame == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG frame passed is NULL */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_writeBack",
                             status,
                             "handle passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);
        status = IFrameQBufMgr_writeBack (obj->handle, frame);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

   GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_writeBack", status);

   /*! @retval FrameQBufMgr_S_SUCCESS  if successfully written back */
   return (status);
}

/*!
 *  @brief  Invalidate the contents  of a frame including frame buffers.
 *
 *          frame  address in virtual format.
 *          It converts the internal portable frame buffer pointers to virtual
 *          format in the frame.
 *  @param  handle  Handle to the instance.
 *  @param  frame   Frame to be writeback.
 */
Int32
FrameQBufMgr_invalidate(FrameQBufMgr_Handle  handle,
                        FrameQBufMgr_Frame   frame)
{
    Int32       status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj  *obj;

    GT_2trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_invalidate",
               handle,
               frame);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (frame  != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval 0 Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_invalidate",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status =  FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_invalidate",
                             status,
                             "handle passed is NULL!");
    }
    else if (frame == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG frame passed is NULL */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_invalidate",
                             status,
                             "handle passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);
        status = IFrameQBufMgr_invalidate (obj->handle, frame);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

   GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_invalidate", status);

   /*! @retval FrameQBufMgr_S_SUCCESS  if successfully invalidates frame */
   return (status);
}

/*!
 *  @brief  Write back the contents  of  frame header.
 *
 *          API treates the passed buffer as frame Header.
 *
 *  @param  handle      Handle to the instance.
 *  @param  headerBuf   frame header to be written back.
 */
Int32
FrameQBufMgr_writeBackHeaderBuf(FrameQBufMgr_Handle  handle,
                                Ptr                  headerBuf)
{
    Int32               status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj   *obj;

    GT_2trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_writeBackHeaderBuf",
               handle,
               headerBuf);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (headerBuf  != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval 0 Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_writeBackHeaderBuf",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status =  FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_writeBackHeaderBuf",
                             status,
                             "handle passed is NULL!");
    }
    else if (headerBuf == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG frame header  passed is NULL */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_writeBackHeaderBuf",
                             status,
                             "frame header passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);
        status = IFrameQBufMgr_writeBackHeaderBuf (obj->handle, headerBuf);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

   GT_1trace (curTrace,
              GT_LEAVE,
              "FrameQBufMgr_writeBackHeaderBuf",
              status);

   /*! @retval FrameQBufMgr_S_SUCCESS  if successfully write back doen for frame
    * header
    */
   return (status);
}

/*!
 *  @brief  Write back the contents  of  frame header.
 *
 *          API treates the passed buffer as frame Header.
 *
 *  @param  handle      Handle to the instance.
 *  @param  headerBuf   frame header to be written back.
 */
Int32
FrameQBufMgr_invalidateHeaderBuf(FrameQBufMgr_Handle  handle,
                                 Ptr                  headerBuf)
{
    Int32               status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj   *obj;

    GT_2trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_invalidateHeaderBuf",
               handle,
               headerBuf);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (headerBuf  != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval 0 Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_invalidateHeaderBuf",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status =  FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_invalidateHeaderBuf",
                             status,
                             "handle passed is NULL!");
    }
    else if (headerBuf == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG frame header  passed is NULL */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_invalidateHeaderBuf",
                             status,
                             "frame header passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);
        status = IFrameQBufMgr_invalidateHeaderBuf (obj->handle, headerBuf);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

   GT_1trace (curTrace,
              GT_LEAVE,
              "FrameQBufMgr_invalidateHeaderBuf",
              status);

   /*! @retval FrameQBufMgr_S_SUCCESS  if successful invalidation done for frame
    * header
    */
   return (status);
}

/*!
 *  @brief  Write back the contents  of frame buffer.
 *
 *          Passed buffer is treated as frame  buffer.
 *          bufNumInFrame denotes  buffer number in the frame.This is  to
 *          identify the cache flags and cpu access flags for the  buffer.
 *
 *  @param  handle      Handle to the instance.
 *  @param  frameBuf    frame buffer to be written back.
 *  @param  size        Size of buffer.
 *  @param  bufIndexInFrame index of the frame buffer in the frame that this
 *          frame buffer belongs to.
 */
Int32
FrameQBufMgr_writeBackFrameBuf(FrameQBufMgr_Handle  handle,
                               Ptr                  frameBuf,
                               UInt32               size,
                               UInt8                bufIndexInFrame)
{
    Int32               status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj   *obj;

    GT_4trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_writeBackFrameBuf",
               handle,
               frameBuf,
               size,
               bufIndexInFrame);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (frameBuf  != NULL));
    GT_assert (curTrace, ( 8 > bufIndexInFrame));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval 0 Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_writeBackFrameBuf",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status =  FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_writeBackFrameBuf",
                             status,
                             "handle passed is NULL!");
    }
    else if (frameBuf == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG frame buffer  passed is NULL */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_writeBackFrameBuf",
                             status,
                             "frame buffer passed is NULL!");
    }
    else if (bufIndexInFrame >= 8) {
        /*! @retval FrameQBufMgr_E_INVALIDARG bufIndexInFrame is greater than 8.
         * frame supports max 8 frame buffers 0 - 7
         */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_writeBackFrameBuf",
                             status,
                             "frame buffer passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);
        status = IFrameQBufMgr_writeBackFrameBuf (obj->handle,
                                                  frameBuf,
                                                  size,
                                                  bufIndexInFrame);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

   GT_1trace (curTrace,
              GT_LEAVE,
              "FrameQBufMgr_writeBackFrameBuf",
              status);

   /*! @retval FrameQBufMgr_S_SUCCESS  if successfully write back done for frame
    * buffer.
    */
   return (status);
}

/*!
 *  @brief  Invalidate the contents  of frame buffer.
 *
 *          Passed buffer is treated as frame  buffer.
 *          bufNumInFrame denotes  buffer number in the frame.This is  to
 *          identify the cache flags and cpu access flags for the given buffer.
 *
 *  @param  handle      Handle to the instance.
 *  @param  frameBuf    frame buffer to be invalidated.
 *  @param  size        Size of buffer.
 *  @param  bufIndexInFrame index of the frame buffer in the frame that this
 *          frame buffer belongs to.
 */
Int32
FrameQBufMgr_invalidateFrameBuf(FrameQBufMgr_Handle  handle,
                                Ptr                  frameBuf,
                                UInt32               size,
                                UInt8                bufIndexInFrame)
{
    Int32               status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj   *obj;

    GT_4trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_invalidateFrameBuf",
               handle,
               frameBuf,
               size,
               bufIndexInFrame);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (frameBuf  != NULL));
    GT_assert (curTrace, ( 8 > bufIndexInFrame));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval 0 Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_invalidateFrameBuf",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status =  FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_invalidateFrameBuf",
                             status,
                             "handle passed is NULL!");
    }
    else if (frameBuf == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG frame buffer  passed is NULL */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_invalidateFrameBuf",
                             status,
                             "frame buffer passed is NULL!");
    }
    else if (bufIndexInFrame >= 8) {
        /*! @retval FrameQBufMgr_E_INVALIDARG bufIndexInFrame is greater than 8.
         * frame supports max 8 frame buffers 0 - 7
         */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_invalidateFrameBuf",
                             status,
                             "frame buffer passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);
        status = IFrameQBufMgr_invalidateFrameBuf (obj->handle,
                                                   frameBuf,
                                                   size,
                                                   bufIndexInFrame);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

   GT_1trace (curTrace,
              GT_LEAVE,
              "FrameQBufMgr_invalidateFrameBuf",
              status);

   /*! @retval FrameQBufMgr_S_SUCCESS  if successfully write back done for frame
    * buffer.
    */
   return (status);
}

/*!
 *  @brief  Function to translate address between different types.
 *
 *          see also FrameQBufMgr_AddrType.
 *
 *  @param  handle      Handle to the instance.
 *  @param  dstAddr     Destination address.
 *  @param  dstAddrType Destination address type.
 *  @param  srcAddr     Source address.
 *  @param  srcAddrType Source address type.
 *  @param  bufType index  Source address specified is frame header or frame
 *          buffer.
 */
Int32
FrameQBufMgr_translateAddr (FrameQBufMgr_Handle      handle,
                            Ptr *                    dstAddr,
                            FrameQBufMgr_AddrType    dstAddrType,
                            Ptr                      srcAddr,
                            FrameQBufMgr_AddrType    srcAddrType,
                            FrameQBufMgr_BufType     bufType)
{
    Int32               status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj    *obj;

    GT_5trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_translateAddr",
               dstAddr,
               dstAddrType,
               srcAddr,
               srcAddrType,
               bufType);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (dstAddr  != NULL));
    GT_assert (curTrace, (srcAddr  != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval 0 Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_translateAddr",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status =  FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_translateAddr",
                             status,
                             "handle passed is NULL!");
    }
    else if (srcAddr == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG srcAddr  passed is NULL */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_translateAddr",
                             status,
                             "srcAddr passed is NULL!");
    }
    else if (dstAddr == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG dstAddr  passed is NULL */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_translateAddr",
                             status,
                             "dstAddr passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);
        status = IFrameQBufMgr_translateAddr (obj->handle,
                                              dstAddr,
                                              dstAddrType,
                                              srcAddr,
                                              srcAddrType,
                                              bufType);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

   GT_1trace (curTrace,
              GT_LEAVE,
              "FrameQBufMgr_translateAddr",
              status);

   /*! @retval FrameQBufMgr_S_SUCCESS  if successfully translated the given source
    * address.
    */
   return (status);
}

/*!
 *  @brief  Function to get FrameQbufMgr Id.
 *
 *  @param  handle  Handle to the instance.
 *
 */
UInt32
FrameQBufMgr_getId (FrameQBufMgr_Handle handle)
{
    UInt32 id = 0;
    FrameQBufMgr_Obj  *obj;

    GT_1trace (curTrace, GT_ENTER, "FrameQBufMgr_getId", handle);

    GT_assert (curTrace, (handle != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval 0 Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_getId",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval 0 handle passed is NULL */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_getId",
                             FrameQBufMgr_E_INVALIDARG,
                             "handle passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);
        id = obj->instId;
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

   GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_getId", id);

   /*! @retval id Instance ID */
   return (id);
}

/*!
 *  @brief  Function to get handle from the FrameQBufMgr Id
 *
 *  @param  Id  FrameQBufMgr id.
 *
 */
Ptr
FrameQBufMgr_getHandle (UInt32 Id)
{
    FrameQBufMgr_Handle              handle  = NULL;
    IArg                             key;
    FrameQBufMgr_Obj                *obj ;
    List_Elem                       *elem;
    UInt16                          instId;
    UInt16                          instNo;
    UInt16                          creProcId;

    GT_1trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_getHandle",
               Id);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval NULL Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_getHandle",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        instId = Id & 0xFFFF;
        /* Find out the creator procid */
        creProcId = (  (instId >> FRAMEQBUFMGR_CREATORPROCID_BITOFFSET)
                      & FRAMEQBUFMGR_CREATORPROCID_MASK);
        GT_assert(curTrace,(creProcId < MultiProc_MAXPROCESSORS));
        /* Get the instance no of the instance */
        instNo = ( (instId >> FRAMEQBUFMGR_INSTNO_BITOFFSET)
                  & FRAMEQBUFMGR_INSTNO_MASK);
        GT_assert(curTrace,(instNo < FrameQBufMgr_MAXINSTANCES));

        if (instNo < FrameQBufMgr_MAXINSTANCES) {
            List_traverse (elem, (List_Handle) &FrameQBufMgr_module->objList) {
                if (   ((FrameQBufMgr_Obj *)elem)->instId
                        == instId) {
                    key = IGateProvider_enter (FrameQBufMgr_module->gate);
                    obj = ((FrameQBufMgr_Obj *)elem);
                    /* Check if we have created the GP or not */
                    handle  = obj->top;
                    IGateProvider_leave (FrameQBufMgr_module->gate, key);
                    break;
                }
            }
        }
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_getHandle", handle);

    /*! @retval Valid-handle Operation successful.
     */
    return (handle);
}

/*!
 *  @brief Provides a hook to perform implementation dependent operation
 *
 *  @param  handle  Instance handle.
 *  @param  cmd     Command.
 *  @param  arg     argument.
 *
 */
Int32
FrameQBufMgr_control (FrameQBufMgr_Handle  handle,
                      Int32                cmd,
                      Ptr                  arg)
{
    Int32   status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj                *obj ;

    GT_3trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_control",
               handle,
               cmd,
               arg);

    GT_assert (curTrace, (handle != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval 0 Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_control",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status =  FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_control",
                             status,
                             "handle passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);

        /* Call implementation specific API.
         * Here obj->handle is the upcasted handle.
         */
        status = IFrameQBufMgr_control(obj->handle,
                                       cmd,
                                       arg);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /*#if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_control", status);
    return (status);
}

/*
 *  ======== FrameQBufMgr_getBaseHeaderSize ========
 *  API to get the base frame heade if number of frame buffers known.
 */
UInt32 FrameQBufMgr_getBaseHeaderSize(UInt8 numFrameBufs)
{
    UInt32 baseHdrSize;

    GT_assert (curTrace, (numFrameBufs <= 8));

    baseHdrSize = (  sizeof(FrameQBufMgr_FrameHeader)
                   + (sizeof(FrameQBufMgr_FrameBufInfo)* (numFrameBufs-1)));

    return (baseHdrSize);
}
/*
 *  ======== FrameQBufMgr_getInstNo ========
 *  API to get the unique instane no for the instances of FrmaeQBufMgr
 *  instances.
 */
Int32 FrameQBufMgr_getInstNo(UInt8 *instNo)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;
    UInt32 i;
    IArg   key;

    GT_1trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_getInstNo",
               instNo);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval FrameQBufMgr_E_INVALIDSTATE Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_getInstNo",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        /* Get unique id for the instance */
        key = IGateProvider_enter (FrameQBufMgr_module->gate );
        for (i = 0; i < FrameQBufMgr_MAXINSTANCES; i++) {
             if (FrameQBufMgr_module->isInstNoValid[i] == FALSE) {
                FrameQBufMgr_module->isInstNoValid[i] = TRUE;
                /* found the  empty slot. This is the unique
                 * instnace no for this instance on this processor
                 *. By appending creator proc id we get the unique
                  * instance id for this instance in the entire system
                */
                *instNo = i;
                break;
             }
        }
        IGateProvider_leave (FrameQBufMgr_module->gate, key);

        if (i == FrameQBufMgr_MAXINSTANCES) {
            status = FrameQBufMgr_E_MAXINSTANCES;
        }
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif
    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_getInstNo", status);
    return (status);
}

/*
 *  ======== FrameQBufMgr_freeInstNo ========
 *  API to free the instance no to the FrameQBufMgr module.
 */
Int32 FrameQBufMgr_freeInstNo(UInt8 instNo)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;
    IArg   key;

    GT_1trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_freeInstNo",
               instNo);

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval FrameQBufMgr_E_INVALIDSTATE Module was not initialized */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_freeInstNo",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        if (instNo >= FrameQBufMgr_MAXINSTANCES) {
            status = FrameQBufMgr_E_FAIL;
        }
        else {
            /* Get unique id for the instance */
            key = IGateProvider_enter (FrameQBufMgr_module->gate );

            FrameQBufMgr_module->isInstNoValid[instNo] = FALSE;

            IGateProvider_leave (FrameQBufMgr_module->gate, key);
        }
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_freeInstNo", status);

    return (status);
}

/*
 *  ======== FrameQBufMgr_getNumFreeFrames ========
 *  Function to get the number of free frames available in
 *  FrameQbufMgr's FreeFramePool 0. The returned number free frame may not
 *  exist in the frameQbufMgr  after this call.This is because
 *  if any allocs are done by the other FrameQ writers on the
 *  same FrameQBufMgr.If frees are not other frameQ writers the free Frames
 *  would be even more than the value that we have got.
 *
 */
Int32
FrameQBufMgr_getNumFreeFrames (FrameQBufMgr_Handle   handle,
                               UInt32              * numFreeFrames)
{
    Int32   status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj                *obj ;

    GT_2trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_getNumFreeFrames",
               handle,
               numFreeFrames);

    GT_assert (curTrace, (handle != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval 0 Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_getNumFreeFrames",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status =  FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_getNumFreeFrames",
                             status,
                             "handle passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);

        /* Call implementation specific API.
         * Here obj->handle is the upcasted handle.
         */
        status = IFrameQBufMgr_getNumFreeFrames (obj->handle, numFreeFrames);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /*#if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_getNumFreeFrames", status);

    return (status);
}
/*
 *  ======== FrameQBufMgr_getvNumFreeFrames ========
 * Function to get the number of free frames available in the plugged in
 * FrameQbufMgr.
 */
Int32
FrameQBufMgr_getvNumFreeFrames (FrameQBufMgr_Handle   handle,
                                UInt32                numFreeFrames[],
                                UInt8                 freeFramePoolNo[],
                                UInt8                 numFreeFramePools)
{
    Int32   status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj                *obj ;

    GT_3trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_getvNumFreeFrames",
               handle,
               numFreeFrames,
               freeFramePoolNo);

    GT_assert (curTrace, (handle != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        /*! @retval 0 Module was not initialized */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_getvNumFreeFrames",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (handle == NULL) {
        /*! @retval FrameQBufMgr_E_INVALIDARG handle passed is NULL */
        status =  FrameQBufMgr_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_getvNumFreeFrames",
                             status,
                             "handle passed is NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        obj = handle->obj;
        GT_assert (curTrace, obj != NULL);

        /* Call implementation specific API.
         * Here obj->handle is the upcasted handle.
         */
        status = IFrameQBufMgr_getvNumFreeFrames (obj->handle,
                                                  numFreeFrames,
                                                  freeFramePoolNo,
                                                  numFreeFramePools);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /*#if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "FrameQBufMgr_getvNumFreeFrames", status);

    return (status);
}

/*=============================================================================
 * Internal Functions
 *=============================================================================
 */
Void FrameQBufMgr_Params_init (FrameQBufMgr_Handle  handle,
                               FrameQBufMgr_Params * params)
{
    FrameQBufMgr_Obj   *obj;

    GT_2trace (curTrace,
               GT_ENTER,
               "FrameQBufMgr_Params_init",
               handle,
               params);

    GT_assert (curTrace, (params != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_Params_init",
                             FrameQBufMgr_E_INVALIDSTATE,
                             "Module was not initialized!");
    }
    else if (params == NULL) {
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "FrameQBufMgr_Params_init",
                             FrameQBufMgr_E_FAIL,
                             "Argument of type (FrameQBufMgr_Params *) is "
                             "NULL!");
    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */
        if (handle == NULL) {
            Memory_copy (params,
                         &(FrameQBufMgr_module->defaultInstParams),
                         sizeof (FrameQBufMgr_Params));

        }
        else {
            obj  = handle->obj;
            Memory_copy ((Ptr)params,
                         (Ptr)&obj->params,
                         sizeof (FrameQBufMgr_Params));
        }
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_0trace (curTrace, GT_LEAVE, "FrameQBufMgr_Params_init");

}
/*!
 * @brief   Internal function called by FrameQBufMgr_Instance_init function when
 *          openFlag is  FALSE.
 *
 * @param   object FrameQbufMgr handle(uncooked)
 */
static inline
Int32
_FrameQBufMgr_create (FrameQBufMgr_Handle  handle,
                      FrameQBufMgr_Params* params)
{
    Int32                         status    = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_ShMem_Handle     shMemHandle;
    FrameQBufMgr_ShMem_Params    *pShMemParams;
    FrameQBufMgr_Obj             *obj;
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    FrameQBufMgr_NameServerEntry  entry;
    UInt32                        len;
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_2trace (curTrace, GT_ENTER, "_FrameQBufMgr_create", handle,params);

    GT_assert (curTrace, (handle != NULL));
    GT_assert (curTrace, (params != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (   Atomic_cmpmask_and_lt (&(FrameQBufMgr_module->refCount),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(0),
                                  FRAMEQBUFMGR_MAKE_MAGICSTAMP(1))
        == TRUE) {
         /*! @retval NULL  Module was not initialized!
          */
        status = FrameQBufMgr_E_INVALIDSTATE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "_FrameQBufMgr_create",
                             status,
                             "Module was not initialized!");
    }
    else if ( NULL == handle) {
         /*! @retval NULL  Params passed is NULL
          */
         status = FrameQBufMgr_E_INVALIDARG;
         GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "_FrameQBufMgr_create",
                             status,
                             "obj of FrameQBufMgr_Object * is NULL!");

    }
    else if ( NULL == params) {
         /*! @retval NULL  Params passed is NULL
          */
         status = FrameQBufMgr_E_INVALIDARG;
         GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "_FrameQBufMgr_create",
                             status,
                             "Params pointer is NULL!");

    }
    else {
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

#if !defined(SYSLINK_BUILD_OPTIMIZE)
        /* Check if name already exists in the name server */
        if (FrameQBufMgr_module->nameServer != NULL) {
            len = sizeof (FrameQBufMgr_NameServerEntry);
            status = NameServer_get (FrameQBufMgr_module->nameServer,
                                     (String)params->commonCreateParams.name,
                                     &entry,
                                     &len,
                                     NULL);
            if (status >= 0) {
                /* Already instance with the name exists */
                /*! @retval NULL  Instance is already created and entry exists
                 *  in Nameserver.
                 */

                status = FrameQBufMgr_E_INST_EXISTS;
                GT_setFailureReason (curTrace,
                                     GT_4CLASS,
                                     "_FrameQBufMgr_create",
                                     status,
                                     "Instance is already created and entry"
                                     "exists in Nameserver.!");
            }
            else if ((status != NameServer_E_NOTFOUND) && (status < 0)) {
                /* Error happened in NameServer. Pass the error up. */
                /*! @retval NULL  IError happened in NameServer.
                 */
                status = FrameQBufMgr_E_FAIL;
                GT_setFailureReason (curTrace,
                                     GT_4CLASS,
                                     "_FrameQBufMgr_create",
                                     status,
                                     "Error happened in NameServer.!");
            }
            else {
                /* Go ahead and create as there is no instance with that name
                 * exists.
                 */
                status = FrameQBufMgr_S_SUCCESS;
            }
        }
#endif
        if (status >= 0) {
            /* Based on the type of interface  mentioned  call create of
             * specific implementation.Currently it has support for FrameQ on
             * shared memory. Need to extend this switch case  to add support
             * for other implementations.
             */
            switch (params->commonCreateParams.ctrlInterfaceType) {
                case FrameQBufMgr_INTERFACE_SHAREDMEM:
                {
                    pShMemParams = (FrameQBufMgr_ShMem_Params *)params;
#if !defined(SYSLINK_BUILD_OPTIMIZE)
                    /* Check the size  of params */
                    if(   pShMemParams->commonCreateParams.size
                        < sizeof(FrameQBufMgr_ShMem_Params)) {
                        /*! @retval NULL  Params passed is NULL
                         */
                        status = FrameQBufMgr_E_INVALIDARG;
                        GT_setFailureReason (curTrace,
                                             GT_4CLASS,
                                             "_FrameQBufMgr_create",
                                             status,
                                             "ommonCreateParams"
                                             ".size is not equal size of"
                                             "FrameQBufMgr_ShMem_Params!");
                    }
                    else if( !FrameQBufMgr_isSupportedBufInterface(params->commonCreateParams.ctrlInterfaceType,
                                                             pShMemParams->bufInterfaceType)) {
                        /*! @retval FrameQBufMgr_E_INVALID_BUFINTERFACETYPE
                         * Unsupported buf interface type.
                         */
                        status = FrameQBufMgr_E_INVALID_BUFINTERFACETYPE;
                        GT_setFailureReason (curTrace,
                                             GT_4CLASS,
                                             "_FrameQBufMgr_create",
                                             status,
                                             "Unsupported buf interface type.");
                    }
                    else {
#endif
                        /* Create the shared memory based FrameQ  instance.*/
                        shMemHandle = FrameQBufMgr_ShMem_create(pShMemParams);
                        if (shMemHandle == NULL) {
                            /*! @retval NULL  Failed to create internal instance of
                             *  type FrameQBufMgr_ShMem .
                             */
                            status = FrameQBufMgr_E_FAIL;
                            GT_setFailureReason (curTrace,
                                                 GT_4CLASS,
                                                 "_FrameQBufMgr_create",
                                                 status,
                                                 "Failed to create internal instance"
                                                 "of type FrameQBufMgr_ShMem!");
                        }
                        else {
                            /* Upcast the handle so that we can call Implementation
                             * specific API using IFrameQBufMgr.
                             */
                            obj =  handle->obj;
                            obj->handle        = shMemHandle;
                            obj->ctrlInterfaceType =
                                       params->commonCreateParams.ctrlInterfaceType;
                            obj->objType       = FrameQBufMgr_DYNAMIC_CREATE;
                            /*  Get the instId  to update the toplevel object */
                            obj->instId        =
                                              FrameQBufMgr_ShMem_getId(shMemHandle);
                            obj->refCount      = 1;
                            obj->notifyId = (UInt32)-1;

                       }
#if !defined(SYSLINK_BUILD_OPTIMIZE)
                    }
#endif
                }
                break;

                default:
                    /*! @retval NULL  invalid interface type
                     */
                    status = FrameQBufMgr_E_INVALID_INTERFACE;
                    GT_setFailureReason (curTrace,
                                         GT_4CLASS,
                                         "_FrameQBufMgr_create",
                                         status,
                                         "invalid interface type!");
            }
        }
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "_FrameQBufMgr_create", status);

    return (status);
}

/*!
 * @brief   Internal function called by FrameQBufMgr_create function when
 *          openFlag is  TRUE.
 *
 * @param   obj FrameQbufMgr internal object.
 */
static inline
Int32
 _FrameQBufMgr_open (FrameQBufMgr_Object* obj,
                     FrameQBufMgr_Params* params)
{

    Int32                status = FrameQBufMgr_S_SUCCESS;

    GT_assert(curTrace,(NULL != obj));

    GT_assert(curTrace,(NULL != params));

    return(status);
}

/*!
 * @brief API to to find out the specified interface type is supported as buffer
 *  interface or not.
 * @param type  interface type.
 */
inline
Int32 FrameQBufMgr_isSupportedBufInterface(UInt32 ctrlInterfaceType,UInt32 bufType)
{
    Int32  status = TRUE;

    switch (ctrlInterfaceType)
    {
        case FrameQBufMgr_INTERFACE_SHAREDMEM:

            switch (bufType)
            {
                /* HLOS side only FrameQBufMgr_BUFINTERFACE_SHAREDMEM is
                 * supported.
                 */
            case FrameQBufMgr_BUFINTERFACE_SHAREDMEM:
                /* Supported interface */
            break;
            default:
                /* unsupported buf interface type specified */
                status = FALSE;
            }

        break;
        default:
            /* unsupported interface type specifiec */
            status = FALSE;
    }

    return (status);
}

/*!
 * @brief API to to find out the specified interface type is supported or not.
 * @param type  interface type.
 */
inline
Int32 FrameQBufMgr_isSupportedInterface(UInt32 type)
{
    Int32  status = TRUE;

    switch (type)
    {
        case FrameQBufMgr_INTERFACE_SHAREDMEM:
            /* Supported interface */
        break;
        default:
            /* unsupported interface type specifiec */
            status = FALSE;
    }

    return (status);
}

/*
 *  ======== FrameQBufMgr_isCacheEnabledForHeaderBuf ========
 * Function to find out if cache is enabled for header buffers.
 */
Bool FrameQBufMgr_isCacheEnabledForHeaderBuf(FrameQBufMgr_Handle handle)
{
    Bool cacheEnabled = FALSE;
    FrameQBufMgr_Obj      *obj;

    GT_1trace (curTrace,
               GT_ENTER,
              "FrameQBufMgr_isCacheEnabledForHeaderBuf",
               handle);

    GT_assert (curTrace, (NULL != handle));
    GT_assert (curTrace, (NULL != handle->obj));


    obj = (FrameQBufMgr_Obj*)handle->obj;

    cacheEnabled =  IFrameQBufMgr_isCacheEnabledForHeaderBuf(obj->handle);

    return (cacheEnabled);
}

/*
 *  ======== FrameQBufMgr_isCacheEnabledForFrameBuf ========
 * Function to find out if cache is enabled for frame buffers.
 */
Bool FrameQBufMgr_isCacheEnabledForFrameBuf(FrameQBufMgr_Handle handle,
                                            UInt8 frameBufIndex)
{
    Bool cacheEnabled = FALSE;
    FrameQBufMgr_Obj      *obj;

    GT_1trace (curTrace,
               GT_ENTER,
              "FrameQBufMgr_isCacheEnabledForFrameBuf",
               handle);

    GT_assert (curTrace, (NULL != handle));
    GT_assert (curTrace, (NULL != handle->obj));

    GT_assert (curTrace, (FrameQBufMgr_MAX_FRAMEBUFS > frameBufIndex));

    obj = (FrameQBufMgr_Obj*)handle->obj;

    cacheEnabled =  IFrameQBufMgr_isCacheEnabledForFrameBuf(obj->handle,
                                                            frameBufIndex);

    return (cacheEnabled);
}

/*!
 * @brief Function to get the sharedMemBaseAddress of the clientNotifyMgr
 *        instance used for this instance.
 * @param handle      FrameQ instance Handle.
 */
Ptr _FrameQBufMgr_getCliNotifyMgrShAddr (FrameQBufMgr_Handle handle)
{
    FrameQBufMgr_Obj      *obj;
    Ptr                    shAddr;

    GT_1trace (curTrace,
               GT_ENTER,
              "_FrameQBufMgr_getCliNotifyMgrShAddr",
               handle);

    GT_assert (curTrace, (NULL != handle));
    GT_assert (curTrace, (NULL != handle->obj));

    obj = (FrameQBufMgr_Obj*)handle->obj;

    /* Call implementation specific API.
     * Here obj->handle is the upcasted handle.
     */
    shAddr = (Ptr)IFrameQBufMgr_getCliNotifyMgrShAddr(obj->handle);

    GT_1trace (curTrace,
               GT_LEAVE,
               "_FrameQBufMgr_getCliNotifyMgrShAddr",
               shAddr );

    /*! @retval  valid handle  if API is successful*/
    return (shAddr);
}

/*!
 * @brief Function to get the sharedMemBaseAddress of the clientNotifyMgr
 *        instance used for this instance.
 * @param handle      FrameQ instance Handle.
 */
Ptr _FrameQBufMgr_getCliNotifyMgrGateShAddr (FrameQBufMgr_Handle handle)
{
    FrameQBufMgr_Obj      *obj;
    Ptr                    shAddr;

    GT_1trace (curTrace,
               GT_ENTER,
              "_FrameQBufMgr_getCliNotifyMgrGateShAddr",
               handle);

    GT_assert (curTrace,
               (   (NULL != handle)
                && (NULL != handle->obj)));

    obj = (FrameQBufMgr_Obj*)handle->obj;

        /* Call implementation specific API.
     * Here obj->handle is the upcasted handle.
     */
    shAddr = IFrameQBufMgr_getCliNotifyMgrGateShAddr(obj->handle);

    GT_1trace (curTrace,
               GT_LEAVE,
               "_FrameQBufMgr_getCliNotifyMgrGateShAddr",
               shAddr );

    /*! @retval  valid handle  if API is successful*/
    return (shAddr);
}


/*!
 *  @brief     Function to set the notifyid received in userspace during call to
 *             ClientNotifyMgr_register client.
 *  @param     handle  Instance handle.
 *  @param     notifyId  Id to to be set in the object.
 *
 */
Int32
 _FrameQBufMgr_setNotifyId (FrameQBufMgr_Handle handle, UInt32 notifyId)
{
    Int32                   status       = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj        *obj;

    GT_2trace (curTrace,
               GT_ENTER,
              "_FrameQBufMgr_setNotifyId",
               handle,
               notifyId);

    GT_assert (curTrace,
               (NULL != handle));

    obj = (FrameQBufMgr_Obj*)handle->obj;

    status = IFrameQBufMgr_setNotifyId(obj->handle, notifyId);

    GT_1trace (curTrace, GT_LEAVE, "_FrameQBufMgr_setNotifyId", status);

    /*! @retval  FrameQBufMgr_S_SUCCESS  */
    return (status);
}

/*!
 *  @brief     Function to reset the notifyid received in userspace during call to
 *             ClientNotifyMgr_register client.
 *  @param     handle  Instance handle.
 *  @param     notifyId  Id to to be set in the object.
 *
 */
Int32
 _FrameQBufMgr_resetNotifyId (FrameQBufMgr_Handle handle, UInt32 notifyId)
{
    Int32                   status  = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Obj        *obj;

    GT_2trace (curTrace,
               GT_ENTER,
              "_FrameQBufMgr_resetNotifyId",
               handle,
               notifyId);

    GT_assert (curTrace,
               (NULL != handle));

    obj = (FrameQBufMgr_Obj*)handle->obj;

    status = IFrameQBufMgr_resetNotifyId(obj->handle, notifyId);

    GT_1trace (curTrace, GT_LEAVE, "_FrameQBufMgr_resetNotifyId", status);

    /*! @retval  FrameQBufMgr_S_SUCCESS  */
    return (status);
}



#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */


